# This file is generated by generate-proto-ruby. Do not edit.
$:.unshift(File.expand_path('../spamcheck', __FILE__))

require 'spamcheck/version'

require 'spamcheck/spamcheck_pb'

require 'spamcheck/spamcheck_services_pb'

